<?php
/**
 Plugin Name: Feed reading Blogroll
 Plugin URI: http://www.weinschenker.name/plugin-feed-reading-blogroll/
 Description: The plugin improves your blogroll by displaying the freshness and the latest post-title of your bookmarks.
 Version: 1.1.3
 Author: Jan Weinschenker
 Author URI: http://www.weinschenker.name


 Plugin: Copyright 2008  Jan Weinschenker  (email: kontakt@weinschenker.name)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 ==============================================================================
 Feed reading Blogroll uses the great JSMin

 JSMin 1.1.1
 jsmin.php - PHP implementation of Douglas Crockford's JSMin.
 Ryan Grove <ryan@wonko.com>
 copyright 2002 Douglas Crockford <douglas@crockford.com> (jsmin.c)
 copyright 2008 Ryan Grove <ryan@wonko.com> (PHP port)
 license: http://opensource.org/licenses/mit-license.php MIT License
 http://code.google.com/p/jsmin-php/
 ==============================================================================
 ==============================================================================
 Feed reading Blogroll uses the great jQuery TinySort
 jQuery TinySort - A plugin to sort child nodes by (sub) contents or attributes.

 Dual licensed under the MIT and GPL licenses:
 http://www.opensource.org/licenses/mit-license.php
 http://www.gnu.org/licenses/gpl.html
 ==============================================================================

 */
if (version_compare(PHP_VERSION, '5.0.0') === 1) {
	require_once('jsmin-1.1.1-frbr.php');
}
if (!defined('WP_CONTENT_URL'))
define('WP_CONTENT_URL', get_option('siteurl') . '/wp-content');
if (!defined('WP_CONTENT_DIR'))
define('WP_CONTENT_DIR', ABSPATH . 'wp-content');
if (!defined( 'WP_PLUGIN_URL'))
define( 'WP_PLUGIN_URL', WP_CONTENT_URL. '/plugins' );
if (!defined( 'WP_PLUGIN_DIR'))
define('WP_PLUGIN_DIR', WP_CONTENT_DIR . '/plugins');
define('FEEDREADING_JS_FILE','/feedreading_blogroll.js');
define('FEEDREADING_VERSION', '1.1.3');


/**
 * add options actions
 */
add_option('feedreading_blogroll_settings', $data, 'feedreading_blogroll Options');
add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'feedreading_blogroll_addConfigureLink' );

add_action('admin_menu',		'feedreading_blogroll_register_options');
add_action('admin_head', 		'feedreading_blogroll_changelog_admin');
add_action('admin_init', 		'feedreading_blogroll_register_setting');
add_action('widgets_init', 		'widget_init_feedreading_blogroll');
add_action('wp_head', 			'feedreading_blogroll_access_google');
add_action('wp_print_scripts',	'feedreading_blogroll_add_scripts');
add_action('wp_print_styles',	'feedreading_blogroll_add_styles');
add_action('wp_ajax_feedreading_blogroll_generate_javascript_lookup', 'feedreading_blogroll_generate_javascript_ajax');

add_action('add_link','feedreading_blogroll_generate_javascript');
add_action('edit_link','feedreading_blogroll_generate_javascript');
add_action('delete_link','feedreading_blogroll_generate_javascript');
add_action('update_option_feedreading_blogroll_settings', 'feedreading_blogroll_generate_javascript');

/**
 * Register uninstall-hook
 * @since WordPress 2.7
 */
if (function_exists('register_uninstall_hook'))
register_uninstall_hook(__FILE__, 'feedreading_blogroll_uninstall_hook');

/**
 * Register activation-hook
 * @since WordPress 2.7
 */
if (function_exists('register_activation_hook'))
register_activation_hook(__FILE__, 'feedreading_blogroll_generate_javascript' );



/**
 * Load text-domain
 * @since WordPress 2.6
 */
if (function_exists('load_plugin_textdomain')) {
	load_plugin_textdomain('feedreading_blogroll', false, dirname(plugin_basename(__FILE__)));
}

/**
 * Initialize the widget
 */
function widget_init_feedreading_blogroll() {
	// Check for required functions
	$widget_ops = array('classname' => 'widget_feedreading_blogroll', 'description' => __('widget_feedreading_blogroll widget description ') );
	wp_register_sidebar_widget('widget_feedreading_blogroll', 'Live Blogroll', 'widget_feedreading_blogroll', $widget_ops);
	register_widget_control('widget_feedreading_blogroll', 'widget_feedreading_blogroll_control');
}

/**
 * Register Settings and options
 * @since WordPress 2.7
 */
function feedreading_blogroll_register_setting(){
	if (function_exists('register_setting')) {
		register_setting('feedreading_blogroll_settings_group', 'feedreading_blogroll_settings');
	}
}

/**
 * The uninstall-hook for the plugin
 * @since WordPress 2.7
 */
function feedreading_blogroll_uninstall_hook() {
	delete_option('feedreading_blogroll_settings');
}
/**
 * Register with options-page
 */
function feedreading_blogroll_register_options() {
	if (function_exists('add_options_page')) {
		$menutitle = '<img src="' . feedreading_blogroll_get_resource_url('ssprc.gif') . '" alt="" />' . ' ';
		add_options_page($menutitle. 'Feed reading Blogroll', $menutitle. 'Feed reading Blogroll', 8, basename(__FILE__), 'feedreading_blogroll_options_subpanel');
	}
}
/**
 * Add configure-link to plugin-list
 *
 * @since WordPress 2.7
 * @param unknown_type $links
 * @return unknown
 */
function feedreading_blogroll_addConfigureLink($links) {
	$settings_link = '<a href=\'options-general.php?page=feedreading_blogroll.php\'>'.__('Settings').'</a>';
	array_unshift( $links, $settings_link );
	return $links;
}

/**
 * Get link-categories from database
 * @return resultset containing term_id and term_id
 */
function feedreading_blogroll_get_linkcats(){
	global $wpdb;
	$cats = $wpdb->get_results("SELECT $wpdb->terms.term_ID, $wpdb->terms.name FROM $wpdb->terms join $wpdb->term_taxonomy on $wpdb->terms.term_id=$wpdb->term_taxonomy.term_id WHERE $wpdb->term_taxonomy.taxonomy = 'link_category' order by name");
	return $cats;
}

function feedreading_blogroll_order_cats($linkCats, $orderString){
	$orderArray=explode(",", $orderString);
	$orderArray2 = array();
	$orderedCats = array();
	$linkCats2= array();
	$i=1;
	foreach($orderArray as $orderedCat){
		$orderArray2[substr($orderedCat,8)]=$i;
		$i=$i+1;
	}
	$i=1;
	foreach($linkCats as $linkCat){
		$catId=(int) $linkCat->term_ID;
		if(!empty($orderArray2->$catId)){
			$linkCats2[$orderArray2[$catId]]=$linkCat;
		} else {
			$linkCats2[$i+1000000]=$linkCat;
		}
		$i=$i+1;
	}
	ksort($linkCats2);
	return array_values($linkCats2);
}

/**
 * Returns true, if new values are contained in POST-request, else false
 * @return boolean
 */
function feedreading_blogroll_vars_are_set(){
	if (isset($_POST['rebuild_javascript'])) {
		feedreading_blogroll_generate_javascript();
	}

	if (isset($_POST['linkCssClass']) or
	isset($_POST['showLastUpdated']) or
	isset($_POST['showLastPostTitle']) or
	isset($_POST['feedreadingTitle']) or
	isset($_POST['showTitle']) or
	isset($_POST['showIcon']) or
	isset($_POST['defaultIcon']) or
	isset($_POST['googleAPIKey']) or
	isset($_POST['previewButton']) or
	isset($_POST['linkCats']) or
	isset($_POST['groupByLinkCats']) or
	isset($_POST['orderBy']) or
	isset($_POST['orderDirection']) or
	isset($_POST['customCSSDir']) or
	isset($_POST['displayStyle']) or
	isset($_POST['feedDiscovery']) or
	isset($_POST['jsSorting']) or
	isset($_POST['jsSortingTimeout'])
	){
		return TRUE;
	} else {
		return FALSE;
	}
}

/**
 * Enqueue scripts after WordPress has finished loading but before any headers are sent.
 * @since WordPress 2.7
 */
function feedreading_blogroll_add_scripts(){
	if (!is_admin()){
		wp_register_script('feedreading_sorter', WP_PLUGIN_URL.'/feed-reading-blogroll/jquery.tinysort.packed.js', array('jquery'), FEEDREADING_VERSION);
		wp_register_script('feedreading_main', WP_CONTENT_URL.'/feedreading_blogroll.js', array('feedreading_sorter'), FEEDREADING_VERSION);
		wp_enqueue_script('feedreading_main');
	} else if(is_admin()){
		wp_register_script('feedreading_admin_ajax', WP_PLUGIN_URL.'/feed-reading-blogroll/feedreading_blogroll_admin.packed.js', array('jquery-ui-tabs','jquery-ui-sortable'), FEEDREADING_VERSION);
		wp_enqueue_script('feedreading_admin_ajax');
	}
}

/**
 * Images/ Icons in base64-encoding
 * @see http://bueltge.de/test/image2base64/
 * @use function feedreading_blogroll_get_resource_url() for display
 */
if( isset($_GET['resource']) && !empty($_GET['resource'])) {
	# base64 encoding
	$feedreading_resources = array(
		'ssprc.gif' =>
			'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABG'.
			'dBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1h'.
			'Z2VSZWFkeXHJZTwAAAK4SURBVBgZBcFfaFV1AADg7/zO2Zy7A3'.
			'fVebdEQ2yCoeVfmmhQGlRgmCg0sEAoeivvWy/BiigYvoi9hPYg'.
			'EfQYKGVgiIhaaJPSVlC6hU7v9Pon166755577+n7ovRtvZ3LjU'.
			'iUAQQEAAQAQBO3HWrc8HHSudyIwRfLShuJOwgJUUSIiAIhIkSI'.
			'iRAhb3H7Urnz++MSibLSRtKY/s3EuPsH9y4TAnFAIEYUyHN6Fr'.
			'NkA0uOlxMQdxC6WL8bsJusxt+nuPYdrVlE5DkiHk7TtYg2QRtR'.
			'wuMKV79l4hy1Kh0Fnn6dVw/zxBAgIsuoZzQbNEgEhIjHFX7/mh'.
			'CRBBavY92bLFjOcwe4fIRrp0ibNFKyFnWCgID+zez9iu2f8NQu'.
			'ZiY4/QE3zgIb3qW0jjSl3iRr0iBoQ0yEjgJ9q3l2H68cpncFY1'.
			'/wzxlgywHyedQzshZ1goAID37j5PuMHaFWpaObbR/Ss5Szh5id'.
			'prOHNTupN8naNEgEhEAemLurfv20cxfGXElXKi7qc/9en709ty'.
			'z88XNfTixSLBY99JJnJqu2pbFEGyLiXLap7JsT5/2Xd9r31htq'.
			'tZqpqSnHf0p136l6bd9+cRyrVCrGxsbcbKySgJDTt8aFyYbrlX'.
			'/tGR527NP3nJ+c0z8wYO3atSYezbl48KDp6WlbV5Zs27vfyckd'.
			'EiBi7q5fxy8pFAqgvHrGmT8bhoeHFYtFs7OzqtWqo0ePKj8/6B'.
			'cUehdKNJA30TJw/5yfHw2A0cvztNoNo6OjCoUCqNVq8mZm9MRF'.
			'O97ZqnrrpvijQb365w8pDpo/v9vZ8WnpXGrV+i02DW21bNkyrV'.
			'ZLqVQyNDTkhe3bDawY9Nf4VdfGrzyI0pf1dvYb0a/cajE52+Oz'.
			'25t1L3lSx7wu7Zw9d36wMJtzbOkurSjI0rp7lamZBTMPdv4PY0'.
			'MOogadRGMAAAAASUVORK5CYII='.
	'',
	'text-list.gif' =>'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABG'.
'dBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1h'.
'Z2VSZWFkeXHJZTwAAADqSURBVDjLY/j//z8DJZiBKgbkzH9cMH'.
'XX6wcgmiwDQJq3nv/4H0SD+OXl5dlA/L+kpOR/QUHB/+zs7P+p'.
'qan/ExIS/kdGRv4PDg7+T10XDHwgpsx8VNC56eWDkJ675Hmhbf'.
'3zB0uPvP1fuvQpOBDj4uKyIyIi/gcGBv738vL67+zs/N/Gxua/'.
'iYnJf11d3f9qamqogRjQcaugZPHjB66V14ZqINrmXyqIn3bvgX'.
'XeJfK8ANLcv+3lfxAN4hsZGWVra2v/V1FR+S8nJ/dfXFz8v5CQ'.
'0H8eHp7/7Ozs/5mZmVEDEWQzRS6gBAMAYBDQP57x26IAAAAASU'.
'VORK5CYII='.
'');
	if(array_key_exists($_GET['resource'], $feedreading_resources)) {
		$content = base64_decode($feedreading_resources[ $_GET['resource'] ]);
		$lastMod = filemtime(__FILE__);
		$client = ( isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ? $_SERVER['HTTP_IF_MODIFIED_SINCE'] : false );
		// Checking if the client is validating his cache and if it is current.
		if (isset($client) && (strtotime($client) == $lastMod)) {
			// Client's cache IS current, so we just respond '304 Not Modified'.
			header('Last-Modified: '.gmdate('D, d M Y H:i:s', $lastMod).' GMT', true, 304);
			exit;
		} else {
			// Image not cached or cache outdated, we respond '200 OK' and output the image.
			header('Last-Modified: '.gmdate('D, d M Y H:i:s', $lastMod).' GMT', true, 200);
			header('Content-Length: '.strlen($content));
			header('Content-Type: image/' . substr(strrchr($_GET['resource'], '.'), 1) );
			echo $content;
			exit;
		}
	}
}

/**
 * Display Images/ Icons in base64-encoding
 * @return $resourceID
 */
function feedreading_blogroll_get_resource_url($resourceID) {
	return trailingslashit( get_bloginfo('url') ) . '?resource=' . $resourceID;
}


/**
 * Enqueue styles after WordPress has finished loading but before any headers are sent.
 * @since WordPress 2.7
 */
function feedreading_blogroll_add_styles(){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	if(!is_admin()){
		if ($feedreading_blogroll_settings['customCSSDir'] != ''){
			$customStyleFile = $feedreading_blogroll_settings['customCSSDir'].'/feedreading_blogroll.css';
			wp_register_style('feedreading_style', $customStyleFile);
			wp_enqueue_style('feedreading_style');
		} else {
			$defaultStyleFile = WP_PLUGIN_URL.'/feed-reading-blogroll/feedreading_blogroll.css';
			wp_register_style('feedreading_style', $defaultStyleFile);
			wp_enqueue_style('feedreading_style');
		}
	} else 	if(is_admin()){
		$adminStyleFile = WP_PLUGIN_URL.'/feed-reading-blogroll/feedreading_blogroll_admin.css';
		wp_register_style('feedreading_admin_style', $adminStyleFile);
		wp_enqueue_style('feedreading_admin_style');
	}
}


/**
 * Read http-data and set options.
 *
 * @return array with updated plugin-options
 */
function feedreading_blogroll_setOptions() {
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	if (feedreading_blogroll_vars_are_set()) {
		$feedreading_blogroll_settings['linkCssClass']      = trim(strip_tags(stripslashes($_POST['linkCssClass'])));
		$feedreading_blogroll_settings['linkCats']          = $_POST['linkCats'];
		$feedreading_blogroll_settings['groupByLinkCats']   = strip_tags(stripslashes($_POST['groupByLinkCats']));
		$feedreading_blogroll_settings['orderBy']           = strip_tags(stripslashes($_POST['orderBy']));
		$feedreading_blogroll_settings['orderDirection']    = strip_tags(stripslashes($_POST['orderDirection']));
		$feedreading_blogroll_settings['showLastUpdated']   = strip_tags(stripslashes($_POST['showLastUpdated']));
		$feedreading_blogroll_settings['showLastPostTitle'] = strip_tags(stripslashes($_POST['showLastPostTitle']));
		$feedreading_blogroll_settings['feedreadingTitle']  = strip_tags(stripslashes($_POST['feedreadingTitle']));
		$feedreading_blogroll_settings['displayStyle']      = strip_tags(stripslashes($_POST['displayStyle']));
		$feedreading_blogroll_settings['showTitle']         = strip_tags(stripslashes($_POST['showTitle']));
		$feedreading_blogroll_settings['defaultIcon']       = trim(strip_tags(stripslashes($_POST['defaultIcon'])));
		$feedreading_blogroll_settings['showIcon']          = strip_tags(stripslashes($_POST['showIcon']));
		$feedreading_blogroll_settings['googleAPIKey']      = strip_tags(stripslashes($_POST['googleAPIKey']));
		$feedreading_blogroll_settings['previewButton']     = strip_tags(stripslashes($_POST['previewButton']));
		$feedreading_blogroll_settings['customCSSDir']      = trim(strip_tags(stripslashes($_POST['customCSSDir'])));
		$feedreading_blogroll_settings['feedDiscovery']     = trim(strip_tags(stripslashes($_POST['feedDiscovery'])));
		$feedreading_blogroll_settings['jsSorting']         = trim(strip_tags(stripslashes($_POST['jsSorting'])));
		$feedreading_blogroll_settings['jsSortingTimeout']  = trim(strip_tags(stripslashes($_POST['jsSortingTimeout'])));
		$feedreading_blogroll_settings['categoryOrderArray'] = trim(strip_tags(stripslashes($_POST['categoryOrderArray'])));
		update_option('feedreading_blogroll_settings', $feedreading_blogroll_settings);
		$feedreading_blogroll_settings['flash'] = __('Settings saved.');
	}
	return $feedreading_blogroll_settings;
}


/**
 * Render the options-panel with form
 */
function feedreading_blogroll_options_subpanel(){
	if (function_exists('wp_nonce_field') and !function_exists('settings_fields')) {
		if (feedreading_blogroll_vars_are_set())
		check_admin_referer('plugin-feed-reading-blogroll-action_option_panel');
	} else if (function_exists('settings_fields')) {
		if (feedreading_blogroll_vars_are_set())
		check_admin_referer('feedreading_blogroll_settings_group-options');
	}
	$feedreading_blogroll_settings = feedreading_blogroll_setOptions();
	$cats = feedreading_blogroll_get_linkcats();
	$menutitle = '<img src="' . feedreading_blogroll_get_resource_url('ssprc.gif') . '" alt="" />' . ' ';
	if (is_null($feedreading_blogroll_settings['linkCats'])) $feedreading_blogroll_settings['linkCats']='';
	if ($feedreading_blogroll_flash != ''){
		?>
<div id="message" class="updated fade">
<p><?php echo $feedreading_blogroll_flash; ?></p>
</div>
		<?php
	}
	?>
<div class="wrap">
<h2><?php echo $menutitle; ?> Feed Reading Blogroll</h2>

	<?php


	if (!$isHook and is_writable(WP_CONTENT_DIR) and file_exists(WP_CONTENT_DIR.FEEDREADING_JS_FILE) and is_writable(WP_CONTENT_DIR.FEEDREADING_JS_FILE)) { ?>
<div id="message" class="error fade">
<p><?php _e('JavaScript-file <strong>and</strong> content-directory are writable. This is no longer necessary. You should make the content-directory read-only (chmod 755) now for security-reasons.</p><p>This plugin requires <strong>only</strong> JavaScript-file itself to be writable.', 'feedreading_blogroll'); ?></p>
<p><?php echo WP_CONTENT_DIR.FEEDREADING_JS_FILE; ?></p>
</div>
	<?php }

	if (feedreading_is_not_jsfile_writable()) {?>
<div id="message" class="error fade">
<p><?php _e('Content directory and/or JavaScript-file not writable!', 'feedreading_blogroll'); ?></p>
<p><?php echo WP_CONTENT_DIR.FEEDREADING_JS_FILE; ?></p>
</div>
	<?php
	}?>

<ul id="feedreading_blogroll_option_tabs">
	<li><span><a href="#feedreading_blogroll_main_tab"><?php _e('Options'); ?></a></span></li>
	<li><span><a href="#feedreading_blogroll_preview_tab"><?php _e('URL-Check','feedreading_blogroll');?></a></span></li>
	<li><span><a href="#feedreading_blogroll_js_tab"><?php _e('JavaScript','feedreading_blogroll');?></a></span></li>
	<li><span><a href="#feedreading_blogroll_news_tab"><?php _e('News','feedreading_blogroll');?></a></span></li>
	<li><span><a href="#feedreading_blogroll_help_tab"><?php _e('Help');?></a></span></li>
</ul>

<div id="feedreading_blogroll_main_tab">
<h3><?php _e('Options'); ?></h3>
<form id="feedreading_options_form" action="" method="post"><?php
if (function_exists('wp_nonce_field') and !function_exists('settings_fields')) {
	wp_nonce_field('plugin-feed-reading-blogroll-action_option_panel');
} else if (function_exists('settings_fields')) {
	settings_fields('feedreading_blogroll_settings_group');
}
?>
<p class="submit"><input name="submit" class="button-primary"
	value="<?php _e('Save Changes'); ?>" type="submit" /></p>

<table class="form-table">
	<tr>
		<th scope="row" valign="top"><label for="googleAPIKey"> <a
			href="http://code.google.com/apis/ajaxfeeds/signup.html"><?php _e('Google API Key','feedreading_blogroll')?></a>
		</label></th>
		<td><input size="90" id="googleAPIKey" name="googleAPIKey" type="text"
			value="<?php echo $feedreading_blogroll_settings['googleAPIKey']; ?>" />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="feedreadingTitle"> <?php _e('Blogroll Title for sidebar widget','feedreading_blogroll')?>
		</label></th>
		<td>
		<fieldset><input size="90" id="feedreadingTitle"
			name="feedreadingTitle" type="text"
			value="<?php echo $feedreading_blogroll_settings['feedreadingTitle']; ?>" /><br />
		<input id="showTitle" name="showTitle" type="checkbox"
			value="showTitle"
			<?php checked('showTitle', $feedreading_blogroll_settings['showTitle']);?> />
			<?php _e('Display title above blogroll','feedreading_blogroll')?></fieldset>
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="feedDiscovery"> <?php _e('Use Google Feed Discovery','feedreading_blogroll')?></label>
		</th>
		<td><input id="feedDiscovery" name="feedDiscovery" type="checkbox"
			value="feedDiscovery"
			<?php checked('feedDiscovery', $feedreading_blogroll_settings['feedDiscovery']);?> />
		<p class="feedreading_info"><?php _e('Automatic Feed Discovery is a service by Google which will try to find the correct feeds that belong to your bookmarks. When using Feed Discovery, you do not need to save extra feed-URLs along with your bookmarks. If you have saved a feed-URL for a bookmark, the plugin will always use the saved feed for that particular bookmark instead of Google Feed Discovery.' ,'feedreading_blogroll')?></p>
		</td>
	</tr>
</table>
<h3><?php _e('Link categories','feedreading_blogroll'); ?></h3>
<table class="form-table">
	<tr>
		<th scope="row" valign="top"><?php _e('Categories')?></th>
		<td>
			<ul id="categoryOrderList">
			<?php
			$orderedArray = feedreading_blogroll_order_cats((array) $cats, $feedreading_blogroll_settings['categoryOrderArray']);
			foreach ($orderedArray as $orderedCat) {
				$checked = "";
				$inBlogroll = "";
				if(!empty($feedreading_blogroll_settings) and !is_null($feedreading_blogroll_settings['linkCats']))
				if (in_array($orderedCat->name , (array)$feedreading_blogroll_settings['linkCats'])==1) {
					$checked =" checked=\"checked\" ";
					$inBlogroll = " inblogroll";
				}
				 ?>
				<li id="linkcat_<?php echo $orderedCat->term_ID;?>" class="<?php echo $inBlogroll;?>"><img
					src="<?php echo feedreading_blogroll_get_resource_url('text-list.gif');?>" />
				<input type="checkbox" class="categoryCheckbox" name="linkCats[]"
				id="checkbox_<?php echo $orderedCat->term_ID ?>" value="<?php echo($orderedCat->name); ?>"
				<?php echo $checked; ?> /> <label
				for="checkbox_<?php echo $orderedCat->term_ID; ?>"><?php echo($orderedCat->name); ?></label>
				</li>
				<?php } ?>
			</ul>
			<p class="feedreading_info"><?php _e('Add a category to your blogroll by activating its checkbox.','feedreading_blogroll')?></p>
			<p class="feedreading_info"><?php _e('Use drag and drop to reorder the categories, then save the changes with the button above.','feedreading_blogroll')?></p>
			<input id="categoryOrderArray" name="categoryOrderArray" size="256"
			type="hidden" />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="groupByLinkCats"> <?php _e('Group bookmarks by category','feedreading_blogroll')?></label>
		</th>
		<td><input id="groupByLinkCats" name="groupByLinkCats" type="checkbox"
			value="groupByLinkCats"
			<?php checked('groupByLinkCats', $feedreading_blogroll_settings['groupByLinkCats']);?> />
		</td>
	</tr>
</table>
<h3><?php _e('Sorting','feedreading_blogroll'); ?></h3>
<table class="form-table">
	<tr>
		<th scope="row" valign="top"><label for="orderBy"><?php _e('Order bookmarks by','feedreading_blogroll')?></label>
		</th>
		<td>
		<fieldset><select name="orderBy" id="orderBy" style="width: 300px;">
			<option value="id"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'id'); ?>>id</option>
			<option value="url"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'url'); ?>>url</option>
			<option value="name"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'name'); ?>>name</option>
			<option value="target"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'target'); ?>>target</option>
			<option value="description"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'description'); ?>>description</option>
			<option value="owner"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'owner'); ?>>owner</option>
			<option value="rating"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'rating'); ?>>rating</option>
			<option value="rel"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'rel'); ?>>rel</option>
			<option value="notes"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'notes'); ?>>notes</option>
			<option value="rss"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'rss'); ?>>rss</option>
			<option value="length"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'length'); ?>>length</option>
			<option value="rand"
			<?php selected($feedreading_blogroll_settings['orderBy'], 'rand'); ?>>rand</option>
		</select><br />
		<input id="orderDirectionASC" name="orderDirection" type="radio"
			value="ASC"
			<?php checked('ASC', $feedreading_blogroll_settings['orderDirection']);?> /><label
			for="orderDirectionASC"><?php _e('ascending','feedreading_blogroll'); ?></label><br />
		<input id="orderDirectionDESC" name="orderDirection" type="radio"
			value="DESC"
			<?php checked('DESC', $feedreading_blogroll_settings['orderDirection']);?> /><label
			for="orderDirectionDESC"><?php _e('descending','feedreading_blogroll'); ?></label>
		</fieldset>
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="jsSorting"><?php _e('Enable JavaScript-sorting','feedreading_blogroll')?></label>
		</th>
		<td><input id="jsSorting" name="jsSorting" type="checkbox"
			value="jsSorting"
			<?php checked('jsSorting', $feedreading_blogroll_settings['jsSorting']);?> />
		<p class="feedreading_info"><?php _e('JavaScript-sorting allows sorting by &quot;latest update&quot;.','feedreading_blogroll')?></p>
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="jsSortingTimeout"><?php _e('JavaScript-sorting timeout','feedreading_blogroll')?></label>
		</th>
		<td>
		<fieldset><select id="jsSortingTimeout" name="jsSortingTimeout"
			style="width: 300px;">
			<option value=""
			<?php selected($feedreading_blogroll_settings['jsSortingTimeout'], ''); ?>>20
				<?php _e('Seconds','feedreading_blogroll'); ?> (<?php _e('Default'); ?>)</option>
			<option value="30"
			<?php selected($feedreading_blogroll_settings['jsSortingTimeout'], '30'); ?>>30
				<?php _e('Seconds','feedreading_blogroll'); ?></option>
			<option value="40"
			<?php selected($feedreading_blogroll_settings['jsSortingTimeout'], '40'); ?>>40
				<?php _e('Seconds','feedreading_blogroll'); ?></option>
			<option value="50"
			<?php selected($feedreading_blogroll_settings['jsSortingTimeout'], '50'); ?>>50
				<?php _e('Seconds','feedreading_blogroll'); ?></option>
			<option value="60"
			<?php selected($feedreading_blogroll_settings['jsSortingTimeout'], '60'); ?>>60
				<?php _e('Seconds','feedreading_blogroll'); ?></option>
		</select></fieldset>
		<p class="feedreading_info"><?php _e('Try to sort the blogroll with JavaScript for this amount of seconds.','feedreading_blogroll')?></p>
		</td>
	</tr>
</table>
<h3><?php _e('Display Options','feedreading_blogroll'); ?></h3>
<table class="form-table">
	<tr>
		<th scope="row" valign="top"><?php _e('Style','feedreading_blogroll'); ?>
		</th>
		<td>
		<fieldset>
		<table>
			<tr>
				<td><input type="radio" id="minimal" name="displayStyle"
					value="minimal"
					<?php checked('minimal',$feedreading_blogroll_settings['displayStyle']); ?> />
				<label for="minimal"><?php _e('Minimal','feedreading_blogroll')?></label>
				</td>
				<td><?php _e('Displays the blogname and age of latest article.', 'feedreading_blogroll'); ?></td>
			</tr>
			<tr>
				<td><input type="radio" id="classic" name="displayStyle"
					value="classic"
					<?php checked('classic',$feedreading_blogroll_settings['displayStyle']); ?> />
				<label for="classic"><?php _e('Classic','feedreading_blogroll')?></label><br />
				</td>
				<td><?php _e('Displays the blogname and below that in one single line the age of the latest article and its title.', 'feedreading_blogroll'); ?></td>
			</tr>
			<tr>
				<td><input type="radio" id="blogger" name="displayStyle"
					value="blogger"
					<?php checked('blogger',$feedreading_blogroll_settings['displayStyle']); ?> />
				<label for="blogger"><?php _e('Blogger(TM)','feedreading_blogroll')?></label><br />
				</td>
				<td><?php _e('Displays the blogname, the age of the latest article and its title. Each element in its own line.', 'feedreading_blogroll'); ?></td>
			</tr>
			<tr>
				<td><input type="radio" id="banners" name="displayStyle"
					value="banners"
					<?php checked('banners',$feedreading_blogroll_settings['displayStyle']); ?> />
				<label for="banners"><?php _e('Banners','feedreading_blogroll')?></label>
				</td>
				<td><?php _e('The bookmark is displayed as a banner-link. Below the image the age of the latest article and its title are displayed.', 'feedreading_blogroll'); ?></td>
			</tr>
		</table>
		</fieldset>
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="customCSSDir"><?php _e('Custom location for stylesheets','feedreading_blogroll'); ?></label>
		</th>
		<td><input size="60" id="customCSSDir" name="customCSSDir" type="text"
			value="<?php echo $feedreading_blogroll_settings['customCSSDir']; ?>" /><?php echo ' /feedreading_blogroll.css'; ?>
		<br />
		<?php _e('Examples: &quot;http://yourblog.com/wp-content&quot; or &quot;http://yourblog.com&quot; or &quot;http://yourblog.com/wp-content/css&quot;.','feedreading_blogroll'); ?>
		<br />
		<?php
		$customfile = $feedreading_blogroll_settings['customCSSDir'].'/feedreading_blogroll.css';
		if (trim($feedreading_blogroll_settings['customCSSDir'])!= '') {
			// everything is okay
		} else {
			_e('No custom location was specified.', 'feedreading_blogroll');
			echo ' ';
			_e('Using default instead: ','feedreading_blogroll');
			?><br />
		<strong><?php
		echo WP_PLUGIN_URL.'/feed-reading-blogroll/feedreading_blogroll.css';
		?></strong><?php
		}
		?>
		<p class="feedreading_info"><?php _e('If you modify the file feedreading_blogroll.css, it is recommended to move it to another directory (e.g. /wp-content) to prevent it from getting lost during a plugin-update.','feedreading_blogroll') ?></p>
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="defaultIcon"><?php _e('Default icon','feedreading_blogroll');?></label>
		</th>
		<td><input size="90" id="defaultIcon" name="defaultIcon" type="text"
			value="<?php echo $feedreading_blogroll_settings['defaultIcon']; ?>" />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="linkCssClass"><?php _e('CSS class for links','feedreading_blogroll')?></label>
		</th>
		<td><input size="90" id="linkCssClass" name="linkCssClass" type="text"
			value="<?php echo $feedreading_blogroll_settings['linkCssClass']; ?>" />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="showLastUpdated"><?php _e('Show update-information','feedreading_blogroll')?></label>
		</th>
		<td><input id="showLastUpdated" name="showLastUpdated" type="checkbox"
			value="showLastUpdated"
			<?php checked('showLastUpdated', $feedreading_blogroll_settings['showLastUpdated']);?> />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="showLastPostTitle"><?php _e('Show title of latest post','feedreading_blogroll')?></label>
		</th>
		<td valign="top"><input id="showLastPostTitle"
			name="showLastPostTitle" type="checkbox" value="showLastPostTitle"
			<?php checked('showLastPostTitle', $feedreading_blogroll_settings['showLastPostTitle']);?> />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="showIcon"><?php _e('Show icon','feedreading_blogroll')?></label>
		</th>
		<td><input id="showIcon" name="showIcon" type="checkbox"
			value="showIcon"
			<?php checked('showIcon', $feedreading_blogroll_settings['showIcon']);?> />
		</td>
	</tr>
	<tr>
		<th scope="row" valign="top"><label for="previewButton"><?php _e('Enable bookmark preview','feedreading_blogroll')?></label>
		</th>
		<td><input id="previewButton" name="previewButton" type="checkbox"
			value="previewButton"
			<?php checked('previewButton', $feedreading_blogroll_settings['previewButton']);?> />
		</td>
	</tr>
</table>
<p class="submit"><input name="submit" class="button-primary"
	value="<?php _e('Save Changes'); ?>" type="submit" /></p>
</form>
</div>

<div id="feedreading_blogroll_js_tab">
<h3><?php _e('JavaScript'); ?></h3>
<form id="feedreading_rebuild_javascript" action="">
<table class="form-table">
	<tr>
		<th scope="row" valign="top"><label for="jsRebuildSubmit"><?php _e('JavaScript file','feedreading_blogroll'); ?></label></th>
		<td>
		<p class="feedreading_info"><?php _e('This file renders article-titles and date-information into your blogroll. It is rebuild automatically by the plugin each time your bookmarks or the options of this plugin change. You can rebuild it manually by clicking the button below.','feedreading_blogroll'); ?></p>
		<p><code><?php echo WP_PLUGIN_DIR.FEEDREADING_JS_FILE; ?></code></p>
		<p><?php _e('Last rebuild:','feedreading_blogroll'); ?> <span
			id="feedreading_ajax_response"><?php echo date(__('dS F, Y h:i:s a','feedreading_blogroll'), filemtime(WP_CONTENT_DIR.FEEDREADING_JS_FILE)); ?></span></p>
		<input type="hidden" name="rebuild_javascript" />
		<p class="submit"><input type="button" id="jsRebuildSubmit"
			name="jsRebuildSubmit" class="submit"
			onclick="feedreadingBlogrollRebuildJS('<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php', '<?php echo wp_create_nonce("plugin-feed-reading-ajax-action_option_panel") ?>')"
			value="<?php _e('Rebuild file','feedreading_blogroll'); ?>"
			<?php if (feedreading_is_not_jsfile_writable()) { ?>
			disabled="disabled" <?php } ?> /></p>
		</td>
	</tr>
</table>
</form>
</div>

<div id="feedreading_blogroll_preview_tab"><?php widget_feedreading_admin_preview(); ?>
</div>

<div id="feedreading_blogroll_news_tab">
<h3 id="feedreading_blogroll_changelog"><?php _e('News','feedreading_blogroll');?></h3>
<div>
<p><?php _e('You are using version','feedreading_blogroll'); echo ' ' . FEEDREADING_VERSION; ?></p>
<p id="feedreading_blogroll_changelog_body"></p>
</div>
</div>
</div>

<div id="feedreading_blogroll_help_tab">
<h3><?php _e('Help','feedreading_blogroll');?></h3>
<p><a href="http://www.weinschenker.name/plugin-feed-reading-blogroll/"
	target="_blank"><?php _e('Plugin Home','feedreading_blogroll');?></a></p>
<p><a href="http://wordpress.org/extend/plugins/feed-reading-blogroll/"
	target="_blank"><?php _e('Download latest Version','feedreading_blogroll');?></a></p>
<p><a
	href="http://www.weinschenker.name/forum/forum/feed-reading-blogroll"
	target="_blank"><?php _e('Supportforum','feedreading_blogroll');?></a></p>
</div>

			<?php
}

/**
 * render the preview of the link-data, that is necessary to make the plugin work properly
 */
function widget_feedreading_admin_preview(){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$linkCssClass = $feedreading_blogroll_settings['linkCssClass'];
	$showLastUpdated = strcmp($feedreading_blogroll_settings['showLastUpdated'], 'showLastUpdated');
	$showIcon = strcmp($feedreading_blogroll_settings['showIcon'], 'showIcon');
	$defaultIcon = $feedreading_blogroll_settings['defaultIcon'];
	$feedDiscovery = strcmp($feedreading_blogroll_settings['feedDiscovery'], 'feedDiscovery');

	?>
<h3 id="feedreading_blogroll_preview"><?php _e('URL-Check','feedreading_blogroll');?></h3>

<p class="feedreading_info"><?php _e('This section will tell you, if RSS-URLs or image-URLs are missing. Update-information in the blogroll can only be displayed if you have saved the necessary URLs.','feedreading_blogroll');?>
</p>

<table class="form-table">
<?php
// fetch bookmarks from wordpress
$bookmarks = get_bookmarks(feedreading_blogroll_bookmarkArgs(FALSE, ''));

foreach ((array)$bookmarks as $bookmark) {
	$name=$bookmark->link_name;
	$url=$bookmark->link_url;
	$urlFeed=$bookmark->link_rss;
	$image=$bookmark->link_image;
	?>
	<tr>
		<th scope="row" valign="top"><strong><?php echo $name;?></strong></th>
		<td>
			<p class="feedreading_info"><?php _e('Web Address');?>: <a href="<?php echo $url;?>"
				target="_blank"><?php echo $url;?></a></p>
			<p class="feedreading_info"><?php _e('RSS Address');?>: <?php
			if (trim($urlFeed) != ''){
				?><a href="<?php echo $urlFeed;?>" target="_blank"><?php echo $urlFeed;?></a><?php
			} else if($feedDiscovery == 0){
				?><span class="feedreading_okay"><?php echo _e('via Google Feed Discovery','feedreading_blogroll');?></span><?php
			} else {
				?><span class="feedreading_error"><?php _e('Warning','feedreading_blogroll'); echo '! '.__('No RSS address! Please provide a feed-url for this bookmark!','feedreading_blogroll').' ';
				?></span> <a
				href="<?php
				echo '/wp-admin/link.php?link_id='.$bookmark->link_id.'&amp;action=edit#linkadvanceddiv';?>"><?php _e('Edit Link');?></a><?php
			}
			?></p>
			<p class="feedreading_info"><?php _e('Image Address');?>: <?php
			if (trim($image) != ''){
				?><a href="<?php echo $image;?>" target="_blank"><?php echo $image;?></a><?php
			} else {
				?><span class="feedreading_warning"><?php _e('Warning','feedreading_blogroll'); echo '! '.__('No image address! If you have configured a default icon, it will be used.','feedreading_blogroll').' ';
				?></span> <a
				href="<?php
				echo '/wp-admin/link.php?link_id='.$bookmark->link_id.'&amp;action=edit#linkadvanceddiv';?>"><?php _e('Edit Link');?></a><?php
			}
			?></p>
		</td>
	</tr>
	<?php
}
?>
</table>
<?php
}

/**
 * Render the widget-body
 */
function widget_feedreading_blogroll($args) {
	extract($args);
	echo $before_widget;
	widget_feedreading_blogroll_content($before_title, $after_title);
	echo $after_widget;
}


/**
 * Render the widget-control
 */
function widget_feedreading_blogroll_control(){
	?>
<p><a href="options-general.php?page=feedreading_blogroll.php">Feed
Reading Blogroll <?php _e('Options'); ?></a></p>
	<?php
}


/**
 * Shortcut for non-widget-users. This is the actual template-tag.
 */
function feedreading_blogroll() {
	widget_feedreading_blogroll_content('<h2>','</h2>');
}

/**
 * transforms the plugin-options into an argument-string that works with the WordPress-function "get_bookmarks()"
 * @return String with arguments for the WordPress-function "get_bookmarks()"
 */
function feedreading_blogroll_bookmarkArgs($usedForGroupedBookmarks, $cat){
	$args='';
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$feedreadingTitle = $feedreading_blogroll_settings['feedreadingTitle'];
	$linkCats = $feedreading_blogroll_settings['linkCats'];
	$orderBy = $feedreading_blogroll_settings['orderBy'];
	$orderDirection = $feedreading_blogroll_settings['orderDirection'];
	$catOption ='';
	$catDelim ='';
	if (!is_null($linkCats)) {
		foreach ((array)$linkCats as $l){
			$catOption = $catOption.$catDelim.$l;
			$catDelim =',';
		}
	}
	$args='order='.$orderDirection.'&orderby='.$orderBy;
	if (!$usedForGroupedBookmarks){
		// use all linkcats
		$args='category_name='.$catOption.'&'.$args;
	} else {
		// use single linkcat
		$args='category_name='.$cat.'&'.$args;
	}
	return $args;
}

/**
 * render content of widget for the sidebar
 */
function widget_feedreading_blogroll_content($before_title, $after_title){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$groupByLinkCats = strcmp($feedreading_blogroll_settings['groupByLinkCats'],'groupByLinkCats');
	$orderString = $feedreading_blogroll_settings['categoryOrderArray'];
	$orderArray = explode(",", $orderString);
	$linkCats = $feedreading_blogroll_settings['linkCats'];

	// fetch bookmarks from wordpress
	if ( $groupByLinkCats==0){
		if (!is_null($linkCats)) {
			foreach($orderArray as $orderedCat){
				$eachTerm = get_term_by('id',substr($orderedCat,8),'link_category');
				$eachName = $eachTerm->name;
				if (in_array($eachName, $linkCats)){
					$bookmarks = get_bookmarks(feedreading_blogroll_bookmarkArgs(TRUE, $eachName));
					widget_feedreading_blogroll_list($before_title, $after_title, $bookmarks, $eachName);
				}
			}
		}
	} else {
		$bookmarks = get_bookmarks(feedreading_blogroll_bookmarkArgs(FALSE,''));
		widget_feedreading_blogroll_list($before_title, $after_title, $bookmarks, '');
	}
}

/**
 * render the blogroll. This is an HTML-unordered-list (<ul>...</ul>) with list-items (<li>...</li>)
 *
 * this function is called one time for every category by widget_feedreading_blogroll_content()
 *
 * @see widget_feedreading_blogroll_content()
 */
function widget_feedreading_blogroll_list($before_title, $after_title, $bookmarks, $title){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$linkCssClass = $feedreading_blogroll_settings['linkCssClass'];
	$showLastUpdated = strcmp($feedreading_blogroll_settings['showLastUpdated'], 'showLastUpdated');
	$showIcon = strcmp($feedreading_blogroll_settings['showIcon'], 'showIcon');
	$showLastPostTitle = strcmp($feedreading_blogroll_settings['showLastPostTitle'], 'showLastPostTitle');
	$showPreviewButton = strcmp($feedreading_blogroll_settings['previewButton'], 'previewButton');
	$defaultIcon = $feedreading_blogroll_settings['defaultIcon'];
	$showTitle = strcmp($feedreading_blogroll_settings['showTitle'],'showTitle');
	$displayStyle = $feedreading_blogroll_settings['displayStyle'];
	$linkCats = $feedreading_blogroll_settings['linkCats'];
	$jsSorting = strcmp($feedreading_blogroll_settings['jsSorting'],'jsSorting');
	$groupByLinkCats = strcmp($feedreading_blogroll_settings['groupByLinkCats'],'groupByLinkCats');
	$feedDiscovery = strcmp($feedreading_blogroll_settings['feedDiscovery'], 'feedDiscovery');

	if ($title =='') $title = $feedreading_blogroll_settings['feedreadingTitle'];
	if ($showTitle == 0 or $groupByLinkCats==0){
		echo $before_title;
		echo $title;
		echo $after_title;
	}
	$safeTitle='';

	if ( $groupByLinkCats==0){
		$safeTerm= get_term_by('name',$title,'link_category');
		$safeTitle = $safeTerm->term_id;
	}
	?>
<ul id="feedreading_blogroll_<?php echo $safeTitle; ?>">
<?php
foreach ((array)$bookmarks as $bookmark){
	// print bookmark

	if ($displayStyle=='minimal'){
		widget_feedreading_blogroll_style_minimal($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery);
	} else if ($displayStyle=='classic'){
		widget_feedreading_blogroll_style_classic($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery);
	} else if ($displayStyle=='blogger') {
		widget_feedreading_blogroll_style_blogger($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery);
	}else if ($displayStyle=='banners') {
		widget_feedreading_blogroll_style_banners($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery);
	}
}
?>
</ul>
<?php
}

/**
 * The simple display-style with name of target-blog and update-information:
 *
 *  * Blog-name
 *    * updated 3 days ago
 */
function widget_feedreading_blogroll_style_minimal($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery){
	$name=$bookmark->link_name;
	$url=$bookmark->link_url;
	$urlFeed=$bookmark->link_rss;
	$image=$bookmark->link_image;
	$xfn=$bookmark->link_rel;
	?>
<li style="margin-top: 1em !important;" class="feedreading_bookmark"
	id="feedreading_bookmark_<?php echo $bookmark->link_id;?>"><?php if (trim($image) != '' and $showIcon == 0){
		?><img src="<?php echo $image; ?>" class="icon16px" width="16"
	height="16" alt="Icon" />&nbsp;<?php
	} else if (trim($image) == '' and $showIcon == 0 and trim($defaultIcon) != ''){
		?><img src="<?php echo trim($defaultIcon); ?>" class="icon16px"
	width="16" height="16" alt="Icon" />&nbsp;<?php
	}
	?><a id="feedreading_anchor_<?php echo $bookmark->link_id;?>"
	href="<?php echo $url;?>" title="<?php echo $name; ?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>
	rel="external <?php echo $xfn;?>" class="<?php echo $linkCssClass; ?>"><?php echo $name; ?></a><?php
	// if bookmark has feed-info, get update-info and print it
	if ((trim($urlFeed)!='' or $feedDiscovery==0) and ($showLastUpdated == 0 or $showLastPostTitle == 0)) { ?>
<div id="feedreading_info_<?php echo $bookmark->link_id;?>"
	class="lastPublicationDate" style="display: block"><abbr
	title="<?php _e('last change','feedreading_blogroll');?>"
	id="feedreading_previewtoggle_<?php echo $bookmark->link_id;?>"></abbr><abbr
	style="display: none"
	id="frbl_last_posttitle_<?php echo $bookmark->link_id;?>"></abbr></div>
<div style="display: none;"
	id="feedreading_preview_wrap_<?php echo $bookmark->link_id;?>">
<p><small><?php _e('Close preview','feedreading_blogroll');?></small></p>
<div id="feedreading_preview_<?php echo $bookmark->link_id;?>">Loading...</div>
</div>
	<?php
	}
	?></li>
	<?php
}

/**
 * The classic display-style with name of target-blog, update-information and unlinked title of most recent post
 *
 *  * Blog-name
 *    * updated 3 days ago: Title of most recent article
 */
function widget_feedreading_blogroll_style_classic($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery){
	$name=$bookmark->link_name;
	$url=$bookmark->link_url;
	$urlFeed=$bookmark->link_rss;
	$image=$bookmark->link_image;
	$xfn=$bookmark->link_rel;
	?>
<li style="margin-top: 1em !important;" class="feedreading_bookmark"
	id="feedreading_bookmark_<?php echo $bookmark->link_id;?>"><?php if (trim($image) != '' and $showIcon == 0){
		?><img src="<?php echo $image; ?>" class="icon16px" width="16"
	height="16" alt="Icon" />&nbsp;<?php
	} else if (trim($image) == '' and $showIcon == 0 and trim($defaultIcon) != ''){
		?><img src="<?php echo trim($defaultIcon); ?>" class="icon16px"
	width="16" height="16" alt="Icon" />&nbsp;<?php
	}
	?><a id="feedreading_anchor_<?php echo $bookmark->link_id;?>"
	href="<?php echo $url;?>" title="<?php echo $name; ?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>
	rel="external <?php echo $xfn;?>" class="<?php echo $linkCssClass; ?>"><?php echo $name; ?></a><?php
	// if bookmark has feed-info, get update-info and print it
	if ((trim($urlFeed)!='' or $feedDiscovery==0) and ($showLastUpdated == 0 or $showLastPostTitle == 0)) { ?>
<div id="feedreading_info_<?php echo $bookmark->link_id;?>"
	class="lastPublicationDate" style="display: block"><abbr
	title="<?php _e('last change','feedreading_blogroll');?>"
	id="feedreading_previewtoggle_<?php echo $bookmark->link_id;?>"></abbr>:
<abbr style="" id="frbl_last_posttitle_<?php echo $bookmark->link_id;?>"></abbr></div>
<div style="display: none;"
	id="feedreading_preview_wrap_<?php echo $bookmark->link_id;?>">
<p><small><?php _e('Close preview','feedreading_blogroll');?></small></p>
<div id="feedreading_preview_<?php echo $bookmark->link_id;?>">Loading...</div>
</div>
	<?php
	}
	?></li>
	<?php
}

/**
 * The Blogger(TM)-like display-style with name of target-blog, update-information and link to most recent article:
 *
 *  * Blog-name
 *    * Link to most recent article
 *    * updated 3 days ago
 */
function widget_feedreading_blogroll_style_blogger($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$name=$bookmark->link_name;
	$url=$bookmark->link_url;
	$urlFeed=$bookmark->link_rss;
	$image=$bookmark->link_image;
	$xfn=$bookmark->link_rel;
	?>
<li class="feedreading_bookmark"
	id="feedreading_bookmark_<?php echo $bookmark->link_id;?>"><?php if (trim($image) != '' and $showIcon == 0){
		?><img src="<?php echo $image; ?>" class="icon16px" width="16"
	height="16" alt="Icon" />&nbsp;<?php
	} else if (trim($image) == '' and $showIcon == 0 and trim($defaultIcon) != ''){
		?><img src="<?php echo trim($defaultIcon); ?>" class="icon16px"
	width="16" height="16" alt="Icon" />&nbsp;<?php
	}
	?><a id="feedreading_anchor_<?php echo $bookmark->link_id;?>"
	href="<?php echo $url;?>" title="<?php echo $name; ?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>
	rel="external <?php echo $xfn;?>" class="<?php echo $linkCssClass; ?>"><?php echo $name; ?></a><?php
	// if bookmark has feed-info, get update-info and print it
	if ((trim($urlFeed)!='' or $feedDiscovery==0) and ($showLastUpdated == 0 or $showLastPostTitle == 0)) { ?>
<div id="feedreading_info_<?php echo $bookmark->link_id;?>"
	class="lastPublicationDate" style="display: block">
<p class="frbl_last_posttitle"><a href="#"
	id="frbl_last_posttitle_<?php echo $bookmark->link_id;?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>></a></p>
<abbr title="<?php _e('last change','feedreading_blogroll');?>"
	id="feedreading_previewtoggle_<?php echo $bookmark->link_id;?>"></abbr></div>
<div style="display: none;"
	id="feedreading_preview_wrap_<?php echo $bookmark->link_id;?>">
<p><small><?php _e('Close preview','feedreading_blogroll');?></small></p>
<div id="feedreading_preview_<?php echo $bookmark->link_id;?>">Loading...</div>
</div>
	<?php
	}
	?></li>
	<?php
}

/**
 * The banner display-style with banner of target-blog, update-information and link to most recent article:
 *
 *  * Blog-banner (Text-Link if no banner-url is supplied)
 *    * Link to most recent article
 *    * updated 3 days ago
 */
function widget_feedreading_blogroll_style_banners($bookmark,$linkCssClass,$showLastUpdated,$showIcon,$showLastPostTitle,$showPreviewButton,$defaultIcon,$feedDiscovery){
	$name=$bookmark->link_name;
	$url=$bookmark->link_url;
	$urlFeed=$bookmark->link_rss;
	$image=$bookmark->link_image;
	$xfn=$bookmark->link_rel;
	?>
<li class="feedreading_bookmark"
	id="feedreading_bookmark_<?php echo $bookmark->link_id;?>"><a
	id="feedreading_anchor_<?php echo $bookmark->link_id;?>"
	href="<?php echo $url;?>" title="<?php echo $name; ?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>
	rel="external <?php echo $xfn;?>" class="<?php echo $linkCssClass; ?>"><?php if (trim($image) != '' and $showIcon == 0){
		?><img src="<?php echo $image; ?>" alt="<?php echo $name; ?>" /><?php
	} else if (trim($image) == '' and $showIcon == 0 and trim($defaultIcon) != ''){
		?><?php echo $name; ?><?php
	}
	?></a><?php
	// if bookmark has feed-info, get update-info and print it
	if ((trim($urlFeed)!='' or $feedDiscovery==0) and ($showLastUpdated == 0 or $showLastPostTitle == 0)) { ?>
<div id="feedreading_info_<?php echo $bookmark->link_id;?>"
	class="lastPublicationDate" style="display: block">
<p class="frbl_last_posttitle"><a href="#"
	id="frbl_last_posttitle_<?php echo $bookmark->link_id;?>"
	<?php if ($bookmark->link_target != '') {?>
	target="<?php echo $bookmark->link_target; ?>" <?php } ?>></a></p>
<abbr title="<?php _e('last change','feedreading_blogroll');?>"
	id="feedreading_previewtoggle_<?php echo $bookmark->link_id;?>"></abbr></div>
<div style="display: none;"
	id="feedreading_preview_wrap_<?php echo $bookmark->link_id;?>">
<p><small><?php _e('Close preview','feedreading_blogroll');?></small></p>
<div id="feedreading_preview_<?php echo $bookmark->link_id;?>">Loading...</div>
</div>
	<?php
	}
	?></li>
	<?php
}

/**
 * HTML for the blog's header-section, containing the necessary Javascript to access the Google API
 */
function feedreading_blogroll_access_google(){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$googleAPIKey = $feedreading_blogroll_settings['googleAPIKey'];
	$bookmarks = get_bookmarks(feedreading_blogroll_bookmarkArgs(FALSE,''));
	$showLastUpdated = strcmp($feedreading_blogroll_settings['showLastUpdated'], 'showLastUpdated');
	$showLastPostTitle = strcmp($feedreading_blogroll_settings['showLastPostTitle'], 'showLastPostTitle');
	$showPreviewButton = strcmp($feedreading_blogroll_settings['previewButton'], 'previewButton');
	if ($showLastUpdated == 0 or $showLastPostTitle == 0){ ?>
<script
	type="text/javascript"
	src="http://www.google.com/jsapi?key=<?php echo $googleAPIKey;?>"></script>
<script type="text/javascript">/* <![CDATA[ */
          google.load("feeds", "1");
          google.setOnLoadCallback(initializeBR, true);
          /* ]]> */
        </script>
	<?php
	}
}

/**
 * Adds a Javascript to the header in the admin section to display the plugin's changelog
 */
function feedreading_blogroll_changelog_admin() {
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$googleAPIKey = $feedreading_blogroll_settings['googleAPIKey'];?>
<script
	type="text/javascript"
	src="http://www.google.com/jsapi?key=<?php echo $googleAPIKey;?>"></script>
<script type="text/javascript">/* <![CDATA[ */
          google.load("feeds", "1");
          google.setOnLoadCallback(feedreading_initialize);
          /* ]]> */
        </script>
	<?php
}
/**
 * Endpoint for ajax-request that rebuilds the javascript-file
 *
 */
function feedreading_blogroll_generate_javascript_ajax(){
	if (function_exists('check_ajax_referer')) {
		check_ajax_referer('plugin-feed-reading-ajax-action_option_panel');
	}
	feedreading_blogroll_generate_javascript();
	$fileTime = date(__('dS F, Y h:i:s a','feedreading_blogroll'), filemtime(WP_CONTENT_DIR.FEEDREADING_JS_FILE));
	die($fileTime);
}

/**
 * Generate the .js-file and store it in WP_CONTENT_DIR.
 *
 */
function feedreading_blogroll_generate_javascript($isHook = TRUE){
	$feedreading_blogroll_settings = get_option('feedreading_blogroll_settings');
	$googleAPIKey = $feedreading_blogroll_settings['googleAPIKey'];
	$bookmarks = get_bookmarks(feedreading_blogroll_bookmarkArgs(FALSE,''));
	$showLastUpdated = strcmp($feedreading_blogroll_settings['showLastUpdated'], 'showLastUpdated');
	$showLastPostTitle = strcmp($feedreading_blogroll_settings['showLastPostTitle'], 'showLastPostTitle');
	$showPreviewButton = strcmp($feedreading_blogroll_settings['previewButton'], 'previewButton');
	$feedDiscovery = strcmp($feedreading_blogroll_settings['feedDiscovery'], 'feedDiscovery');
	$jsSorting = strcmp($feedreading_blogroll_settings['jsSorting'], 'jsSorting');
	$timeout = $feedreading_blogroll_settings['jsSortingTimeout'];
	$title = $feedreading_blogroll_settings['feedreadingTitle'];
	if ($timeout=="") {
		$timeout="20000";
	} else {
		$timeout=$timeout."000";
	}
	ob_start();
	?>
<script type="text/javascript">/* <![CDATA[ */
			/**
			 * printf() for Javascript
			 */
			function sprintf() {
                if( sprintf.arguments.length < 2 ) {
                    return;
                }
                var data = sprintf.arguments[ 0 ];
                for( var k=1; k<sprintf.arguments.length; ++k ) {
                    switch( typeof( sprintf.arguments[ k ] ) ){
                        case 'string':
                            data = data.replace( /%s/, sprintf.arguments[ k ] );
                            break;
                        case 'number':
                            data = data.replace( /%d/, sprintf.arguments[ k ] );
                            break;
                        case 'boolean':
                            data = data.replace( /%b/, sprintf.arguments[ k ] ? 'true' : 'false' );
                            break;
                            default:
                           /// function | object | undefined
                        break;
                    }
                }
                return(data);
            }

            if( !String.sprintf ) {
            String.sprintf = sprintf;
            }


            /**
             * generate text showing the age of the bookmark's latest post
             */
            function getAge(days, hours){
            	var ageMsg = "";
                if (days > 1) {
                    ageMsg = String.sprintf("<?php _e('%d days ago','feedreading_blogroll') ?>", days);
                } else if (days==1){
                    ageMsg = "<?php _e('yesterday','feedreading_blogroll') ?> ";
                } else if (days< 1 && hours > 1) {
                    ageMsg = String.sprintf("<?php _e('%d hrs ago','feedreading_blogroll') ?>", hours);
                } else if (days< 1 && hours == 1) {
                    ageMsg = "<?php _e('in the last hour','feedreading_blogroll') ?>";
                } else if (days < 1 && hours < 1) {
                    ageMsg="<?php _e('just recently','feedreading_blogroll') ?>";
                }
                return ageMsg;
            }
            /* add jQuery-oberserver to enable mouse-clicks*/
            function addObserver(link, wrapper, preview, feed, name){
                jQuery(link).click(function(){
                    jQuery(wrapper).toggle("slow");
                    var feedControl = new google.feeds.FeedControl();
                    feedControl.addFeed(feed, name);
                    feedControl.draw(jQuery(preview).get(0));
                });
                jQuery(wrapper).click(function(){
                	jQuery(wrapper).toggle("slow");
                });
            }
            /* add age and other info to bookmark*/
            function addAge(entry, bookmark, anchor, divID, previewtoggle, last_posttitle){
                var $li = jQuery(bookmark),
                 $a = jQuery(anchor),
                 $toggle = jQuery(previewtoggle),
                 $title = jQuery(last_posttitle),
                 now = new Date(),
                 then = new Date(),
                 ageInDays,
                 ageInHours,
                 randomAge,
                 ageMsg,
                 $snippet;
                try {
                    then = new Date(entry.publishedDate);
                } catch (dateException) {
                    // do noting
                }

                ageInDays =  Math.floor((now.getTime()-then.getTime())/(1000*60*60*24)),
                ageInHours = Math.floor((now.getTime()-then.getTime())%(1000*60*60*24)/3600000);

                try {
                    $entryTitle = jQuery("<p />").html(entry.title);
                    $a.attr({title: $entryTitle.html()});
                } catch (titleException) {
                    $a.attr({title: ""});
                }
				if (!isNaN(then.getTime())){
					// insert age into list-item to allow sorting by age of post
                    $li.attr({age:then.getTime()});
				} else {
		            /* add "very old" age to bookmarks with no feed-url */
					randomAge = Math.floor(Math.random()*1000001);
					$li.attr({age:randomAge});
				}

                <?php if ($showLastUpdated==0) { ?>
                if (isNaN(ageInDays) || isNaN(ageInHours)){
                    // do nothing
                } else {
                    ageMsg=getAge(ageInDays, ageInHours);
                    $toggle.html(ageMsg);
                }
                <?php } ?>
                <?php if ($showLastPostTitle==0) { ?>
                    $title.html(entry.title);
                    $title.attr({href:entry.link});
                    $snippet = jQuery("<p />").html(entry.contentSnippet);
                    $title.attr({title:$snippet.html()});
                    $title.attr({rel:"external"});
                    $title.attr({rev:"bookmark"});
                <?php } ?>
                return;
            }
            /* add "very old" age to bookmarks with no feed-url */
            function addZeroAge(zeroAgeBookmark){
                var $zali = jQuery(zeroAgeBookmark),
                randomAge = Math.floor(Math.random()*1000001);
                if ($zali !== null ) {
                    $zali.attr({age:randomAge});
                }
                return;
            }

            /*
             *
             */
            function feedreading_automatic_sort() { <?php
            	$groupByLinkCats = strcmp($feedreading_blogroll_settings['groupByLinkCats'],'groupByLinkCats');
            	$linkCats = $feedreading_blogroll_settings['linkCats'];
            	if ( $groupByLinkCats==0){
            		if (!is_null($linkCats)) {
            			foreach ((array)$linkCats as $linkCat){
					$safeTerm = get_term_by('name',$linkCat,'link_category');
            				$safeTitle = $safeTerm->term_id; ?>
            				jQuery("#feedreading_blogroll_<?php echo $safeTitle; ?>>li").tsort({order:"desc",attr:"age"});
            				<?php
             			}
            		}
            	} else {

            		?>
    				jQuery("#feedreading_blogroll_ > li").tsort({order:"desc",attr:"age"});
            		<?php
            }
                 ?>
            }
			<?php if ($jsSorting == 0 ) { ?>
            /* call sort-function every half second */
            var myInterval = window.setInterval(function (){feedreading_automatic_sort(); },500);

            /* stop calling sort-function after 20 seconds */
			window.setTimeout(function (a,b){clearInterval(myInterval);}, <?php echo $timeout; ?>);
			<?php } ?>

            function initializeBR() {
                
                <?php
                	
                	$printVars = true;
                	$printSemicolon=false;
                	$komma ='';
                    foreach ((array)$bookmarks as $bookmark){
                    	$urlFeed=$bookmark->link_rss;
                    	if (trim($urlFeed) != "" ){
                    		
                    		if($printVars){?>
                    		var 
	                    		<?php
	                    		$printVars=false;
	                    		$printSemicolon=true;
	                    		}
                    		echo $komma; ?>
                    		feed<?php echo $bookmark->link_id;?> = new google.feeds.Feed("<?php echo $urlFeed; ?>")
                    		<?php $komma=',';
						}
                    }
                    if($printSemicolon){
                    ?>;<?php
					}
                    foreach ((array)$bookmarks as $bookmark){
                        $name=$bookmark->link_name;
                   		$urlFeed=$bookmark->link_rss;

                        if(trim($bookmark->link_rss)!='') {?>
                                      feed<?php echo $bookmark->link_id;?>.load(function(result_<?php echo $bookmark->link_id;?>){
                                      if (!result_<?php echo $bookmark->link_id;?>.error) {
                                      addAge(result_<?php echo $bookmark->link_id;?>.feed.entries[0], "#feedreading_bookmark_<?php echo $bookmark->link_id;?>", "#feedreading_anchor_<?php echo $bookmark->link_id;?>", "#feedreading_info_<?php echo $bookmark->link_id;?>", "#feedreading_previewtoggle_<?php echo $bookmark->link_id;?>", "#frbl_last_posttitle_<?php echo $bookmark->link_id;?>"); <?php
                                if ($showPreviewButton==0){?>
                                      addObserver("#feedreading_previewtoggle_<?php echo $bookmark->link_id?>", "#feedreading_preview_wrap_<?php echo $bookmark->link_id;?>", "#feedreading_preview_<?php echo $bookmark->link_id;?>", "<?php echo $urlFeed; ?>", "<?php echo $name; ?>");
    						<?php } // end if ?>
                        } else {
                  			addZeroAge("#feedreading_bookmark_<?php echo $bookmark->link_id;?>");
              			}
                                      });
                      <?php } else if ($feedDiscovery==0) { ?>
						google.feeds.lookupFeed("<?php echo $bookmark->link_url;?>", function() {
							var url<?php echo $bookmark->link_id;?>= this.url, feed<?php echo $bookmark->link_id;?> = new google.feeds.Feed(this.url);
                            feed<?php echo $bookmark->link_id;?>.load(function(result_<?php echo $bookmark->link_id;?>){
                                if (!result_<?php echo $bookmark->link_id;?>.error) {
                                addAge(result_<?php echo $bookmark->link_id;?>.feed.entries[0], "#feedreading_bookmark_<?php echo $bookmark->link_id;?>", "#feedreading_anchor_<?php echo $bookmark->link_id;?>", "#feedreading_info_<?php echo $bookmark->link_id;?>", "#feedreading_previewtoggle_<?php echo $bookmark->link_id;?>", "#frbl_last_posttitle_<?php echo $bookmark->link_id;?>"); <?php
                          if ($showPreviewButton==0){?>
                                addObserver("#feedreading_previewtoggle_<?php echo $bookmark->link_id?>", "#feedreading_preview_wrap_<?php echo $bookmark->link_id;?>", "#feedreading_preview_<?php echo $bookmark->link_id;?>", url<?php echo $bookmark->link_id;?>, "<?php echo $name; ?>");
						<?php } // end if ?>
                  			} else {
                      			addZeroAge("#feedreading_bookmark_<?php echo $bookmark->link_id;?>");
                  			}
                                });

						});

                      <?php } else { ?>
                      	addZeroAge("#feedreading_bookmark_<?php echo $bookmark->link_id;?>");
                      <?php }// end if ?>
                    <?php } // end foreach ?>

                }


            /* ]]> */</script>
                    <?php
                    $buffer = ob_get_contents();
                    // remove script-tags, as they are not necessary in a flat .js-file
                    // they are only needed during development to allow automatic code-formatting, d'oh
                    $buffer = str_replace('<script type="text/javascript">/* <![CDATA[ */','', $buffer);
                    $buffer = str_replace('/* ]]> */</script>','', $buffer);

                    if (version_compare(PHP_VERSION, '5.0.0') === 1 and class_exists('JSMinFRBR')) {
                    	$buffer = JSMinFRBR::minify($buffer);
                    }

                    ob_end_clean();

                    if ((is_writable(WP_CONTENT_DIR) and !file_exists(WP_CONTENT_DIR.FEEDREADING_JS_FILE)) or is_writable(WP_CONTENT_DIR.FEEDREADING_JS_FILE)) {
                    	$fp = fopen(WP_CONTENT_DIR.FEEDREADING_JS_FILE, "w");
                    	fwrite($fp, $buffer);
                    	fclose($fp);
                    }
                    return;
}
/**
 * is the JavaScript-file writable?
 *
 * @return boolean
 */
function feedreading_is_not_jsfile_writable(){

	return (!is_writable(WP_CONTENT_DIR) and !file_exists(WP_CONTENT_DIR.FEEDREADING_JS_FILE)) or !is_writable(WP_CONTENT_DIR.FEEDREADING_JS_FILE);
}
?>
